var xmlbuilder = require('xmlbuilder'),
	request = require('request'),
	xml2js = require('xml2js'),
	moment = require('moment');

var Message = require("./message");

/*
https://api.esendex.com/v1.0/messagedispatcher

<?xml version='1.0' encoding='UTF-8'?>  
<messages>  
 <accountreference>EX0000000</accountreference> 
 <sendat>2012-01-01T14:31:00</sendat>
 <message>  
  <to>07700900123</to>  
  <type>SMS</type>  
  <body> This is an SMS message </body>
 </message>  
</messages>


<?xml version="1.0" encoding="utf-8"?> 
<messageheaders batchid="F8BF9867-FF81-49E4-ACC5-774DE793B776" xmlns="http://api.esendex.com/ns/"> 
	<messageheader uri="https://api.esendex.com/v1.0/MessageHeaders/1183C73D-2E62-4F60-B610-30F160BDFBD5" id="1183C73D-2E62-4F60-B610-30F160BDFBD5" /> 
</messageheaders>

*/

var MessageDispatcherResponse = function (batchId, messageIds) {
	this.batchId = batchId;
	
	if (Object.prototype.toString.call(messageIds) !== '[object Array]') {
		if (messageIds !== undefined) {
			this.messageIds = [].concat(messageIds);
		}
	} else {
		this.messageIds = messageIds;
	}
}

var buildMessagesRequestBody = function (xmlbuilder, messages, accountRef, sendAt) {	
	var root = xmlbuilder.create('messages');

	root.element('accountreference', accountRef);
	if (sendAt !== undefined && sendAt != null)
		root.ele('sendat', moment(sendAt).format('YYYY-MM-DDTHH:mm:ss.SSS') + 'Z');
	
	messages.forEach(function (msg, idx) {
		var messageEle = root.ele('message');
		messageEle.ele('to', msg.to);
		messageEle.ele('type', msg.type);
		messageEle.ele('body', msg.body);
	});
	
	return root.end({ pretty: false});
};

var MessageDispatcherService = function (apiHost, session) {

	if (session === undefined || session.sessionId === undefined)
		throw "session not supplied";

	this.apiHost = apiHost;
	this.session = session;
	
	this.dependencies = {
		request: request,
		xml2js: xml2js,
		moment: moment,
		xmlbuilder: xmlbuilder
	};
};
MessageDispatcherService.prototype.send = function (messages, accountRef, callback) {
	if (messages === undefined)
		throw "messages must be supplied";
	if (typeof accountRef !== "string")
		throw "accountRef must be supplied, and a string";
		
	if (Object.prototype.toString.call(messages) !== '[object Array]')
		messages = [].concat(messages);
		
	var $this = this;
	var xml = buildMessagesRequestBody($this.dependencies.xmlbuilder, messages, accountRef);
		
	$this.dependencies.request({
			uri: $this.apiHost + "/v1.0/messagedispatcher",
			headers: { "Authorization": $this.session.toAuthorization() },
			method: "POST",
			body: xml
		},
		function (error, response, body) {
			$this.handleDispatcherResponse(error, response, body, callback);
		}
	);
};
MessageDispatcherService.prototype.schedule = function (messages, accountRef, sendAt, callback) {
	if (messages === undefined)
		throw "messages must be supplied";
	if (typeof accountRef !== "string")
		throw "accountRef must be supplied, and a string";
	if (sendAt === undefined || sendAt == null || sendAt.getMonth === undefined)
		throw "sendAt must be a valid Date";
		
	if (Object.prototype.toString.call(messages) !== '[object Array]')
		messages = [].concat(messages);
		
	var xml = buildXml(messages, accountRef, sendAt);
	var $this = this;
		
	request({
			uri: this.apiHost + "/v1.0/messagedispatcher",
			headers: { "Authorization": this.session.toAuthorization() },
			method: "POST",
			body: xml
		},
		function (error, response, body) {
			$this.handleDispatcherResponse(error, response, body, callback);
		}
	);
};
MessageDispatcherService.prototype.handleDispatcherResponse = function (error, response, body, callback) {
	if (!error && response.statusCode != 200) 
		throw "Dispatcher service returned an error: " + response.statusCode;
	if (error) 
		throw "Dispatcher failed to communicate with API service: " + error;
	
	var parser = new xml2js.Parser();
	parser.on('end', function(result) {
		var messageHeaderIds = [];
		result.messageheaders.messageheader.forEach(function (message) {
			messageHeaderIds.push(message.$.id);
		});
		
		var dispatchResponse = new MessageDispatcherResponse(result.messageheaders.$.batchid, messageHeaderIds);
		
		callback(dispatchResponse);
	});
	
	parser.parseString(body);
};

module.exports = MessageDispatcherService;