var sinon = require('sinon'),
	assert = require('assert');

var xml2js = require('xml2js'),
	request = require('request');
	
var sdk = require('../');

describe('SessionService', function () {
	describe('#create()', function () {
		
		it('should post a request to the session api service', function () {
			var requestSpy = sinon.spy();
			var xml2jsStub = sinon.stub(xml2js, "Parser");
			
			var apiHost = "a host";
		
			var service = new sdk.SessionService(apiHost);
			service.dependencies.request = requestSpy;
			service.dependencies.xml2js = xml2jsStub;
			
			var username = "a user name";
			var password = "a password";
			
			service.create(username, password, sinon.stub());

			sinon.assert.calledWith(requestSpy, sinon.match({ uri: apiHost + "/v1.0/session/constructor" }));
			sinon.assert.calledWith(requestSpy, sinon.match({ method: "POST" }));
			
			xml2jsStub.restore();
		});
		
		it('should pass the callback to the response handler', function () {
			var requestStub = sinon.stub();
			var xml2jsStub = sinon.stub(xml2js, "Parser");
			var callbackSpy = sinon.spy();
			
			requestStub.callsArg(1);
		
			var service = new sdk.SessionService("a host");
			service.dependencies.request = requestStub;
			service.dependencies.xml2js = xml2jsStub;
			sinon.stub(service, "handlerSessionConstructorResponse").callsArg(3);
			
			service.create("a user name", "a password", callbackSpy);

			sinon.assert.calledOnce(callbackSpy);
			
			xml2jsStub.restore();
		});
		
	});
	
	describe('#create() with invalid parameters', function () {
	
		it('should throw an exception when username is undefined', function () {
			
			var service = new sdk.SessionService("a host");
			assert.throws(function () {
				service.create();
			});
		
		});
		
		it('should throw an exception when username is not a string', function () {
			
			var service = new sdk.SessionService("a host");
			assert.throws(function () {
				service.create([]);
			});
		
		});
		
		it('should throw an exception when password is undefined', function () {
			
			var service = new sdk.SessionService("a host");
			assert.throws(function () {
				service.create("username");
			});
		
		});
		
		it('should throw an exception when password is not a string', function () {
			
			var service = new sdk.SessionService("a host");
			assert.throws(function () {
				service.create("username", {});
			});
		
		});
		
		it('should throw an exception when callback is undefined', function () {
			
			var service = new sdk.SessionService("a host");
			assert.throws(function () {
				service.create("username", "password");
			});
		
		});
		
		it('should throw an exception when callback is not a function', function () {
			var requestSpy = sinon.spy(request);
			
			var service = new sdk.SessionService("a host");
			service.dependencies.request = requestSpy;
			
			assert.throws(function () {
				service.create("username", "password", "definitely not a function");
			});
			
			sinon.assert.notCalled(requestSpy);
		
		});
	
	});
	
	describe('#handlerSessionConstructorResponse()', function () {
	
		it('should xml parse the body', function () {
			var parserMock = sinon.mock(new xml2js.Parser());
			parserMock.expects("on").once();
			parserMock.expects("parseString").once();
			
			var xml2jsMock = sinon.mock(xml2js);
			xml2jsMock.expects("Parser").once().returns(parserMock.object);
		
			var service = new sdk.SessionService("a host");
			service.dependencies.xml2js = xml2jsMock.object;
			
			service.handlerSessionConstructorResponse(null, { statusCode: 200 }, "a body", sinon.stub());

			xml2jsMock.verify();
			parserMock.verify();
		});
		
		it('should call the callback with the session ID when parsing is finished', function () {
			var callbackSpy = sinon.spy();
			var sessionId = "a convoluted id";
		
			var service = new sdk.SessionService("a host");
			
			service.handlerSessionConstructorResponse(null, { statusCode: 200 }, "<session><id>" + sessionId + "</id></session>", callbackSpy);
			
			sinon.assert.calledWith(callbackSpy, sinon.match({ sessionId: sessionId }));
		});
	
	});
	
	describe('#handlerSessionConstructorResponse() with invalid parameters', function () {
	
		it('should throw an exception when error is not null', function () {
			var service = new sdk.SessionService();
			
			assert.throws(function () {
				service.handlerSessionConstructorResponse("this is not a null error");
			});
		});
		
		it('should throw an exception when http response is not 200 OK', function () {
			var service = new sdk.SessionService();
			
			assert.throws(function () {
				service.handlerSessionConstructorResponse(null, { statusCode: 401 });
			});
		});
	
	});
});